//
//  SynthesizerOutput.m
//  AnalogSynthesizer
//
//  Created by 成田 拓郎 on 10/05/28.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "SynthesizerOutput.h"


@implementation SynthesizerOutput

@synthesize keyboard;

//初期化
- (id)init
{
	self = [super init];
	if (self != nil)
	{
		// 基本波形計算
		for (int i=0; i<SAMPLE_RATE; ++i) {
			float phase = i * M_PI * 2.0f / SAMPLE_RATE;
			wave[i] = sinf(phase)
					+ sinf(phase * 2.0f) / 2.0f
					+ sinf(phase * 3.0f) / 3.0f
					+ sinf(phase * 4.0f) / 4.0f
					+ sinf(phase * 5.0f) / 5.0f
					+ sinf(phase * 6.0f) / 6.0f
					+ sinf(phase * 7.0f) / 7.0f
					+ sinf(phase * 8.0f) / 8.0f
					+ sinf(phase * 9.0f) / 9.0f
					+ sinf(phase * 10.0f) / 10.0f;
			wave[i] /= 2.0f;
		}
		//再生情報
		for (int i=0; i<25; ++i) {
			outPhase[i] = 0.0f;
			outLevel[i] = 0.0f;
		}
	}
	return self;
}

//サウンド コールバック関数(クラス メンバ)
-(OSStatus)outputCallback:(AudioUnitRenderActionFlags *)ioActionFlags
			  inTimeStamp:(const AudioTimeStamp *)inTimeStamp
			  inBusNumber:(UInt32)inBusNumber
		   inNumberFrames:(UInt32)inNumberFrames
				   ioData:(AudioBufferList *)ioData
{
	OSStatus err = noErr;
	
	//	NSLog(@"mNumberBuffers = %i mNumberChannels = %i", ioData->mNumberBuffers, ioData->mBuffers[0].mNumberChannels);
	float* pBuffer = buffer;
	float* pBufferEnd = buffer + inNumberFrames;
	while (pBuffer != pBufferEnd) {
		*pBuffer = 0.0f;
		++pBuffer;
	}

	UInt32 key = self.keyboard;
	int scale = 69 - 24;
	for (int i=0; i<25; ++i) {
		BOOL flag = (key & (1 << i)) != 0;

		int note_number = scale + i;
		float freqe = 440.0f * powf(2.0f, (note_number - 69.0f) / 12.0f);

		float phase = outPhase[i];
		float level = outLevel[i];
		pBuffer = buffer;
		while (pBuffer != pBufferEnd) {
			if (flag) level += 0.001f;
			else	  level -= 0.001f;

			if (level > 1.0f) level = 1.0f;
			else if (level <= 0.0f) {
				level = 0.0f;
				break;
			}

			*pBuffer += wave[(int)phase] * level;
			phase += freqe;
			if (phase >= SAMPLE_RATE)
				phase -= SAMPLE_RATE;
			++pBuffer;
		}
		outPhase[i] = phase;
		outLevel[i] = level;
	}

	// オーディオ バッファに書き込み
	for (int i=0; i < ioData->mNumberBuffers; ++i)
	{
		AudioSampleType *ptr = ioData->mBuffers[i].mData;
		for (int j=0; j < inNumberFrames; ++j)
		{
			float sinewave = buffer[j];
			
			if (sinewave < -1.0)
				sinewave = -1.0;
			else if (sinewave > 1.0)
				sinewave = 1.0;
			sinewave = sinewave * INT16_MAX;
//			sinewave = sinewave * (1 << kAudioUnitSampleFractionBits);
			
			UInt32 channels = ioData->mBuffers[i].mNumberChannels;
			for (int k=0; k < channels; ++k)
			{
				ptr[j * channels + k] = sinewave;
			}
		}
	}
	
	return err;
}

@end
